DETAIL_INFO = {
    "id": 3,
    "name": "АО Рога и копыта",
    "responsible_user_id": 504141,
    "group_id": 0,
    "created_by": 504141,
    "updated_by": 504141,
    "created_at": 1582117331,
    "updated_at": 1586361223,
    "closest_task_at": None,
    "custom_fields_values": [
        {
            "field_id": 3,
            "field_name": "Телефон",
            "field_code": "PHONE",
            "field_type": "multitext",
            "values": [{"value": "123213", "enum_id": 1, "enum": "WORK"}],
        }
    ],
    "account_id": 10,
    "_links": {"self": {"href": "https://exmaple.amocrm.ru/api/v4/companies/1"}},
    "_embedded": {"tags": []},
}
